#!/usr/bin/ruby
# coding: utf-8

@gm = 5000
@gn = 200
@gr = 7
@vs1 = []
@vs2 = [[15, 38], [213, 18], [239, 69], [443, 6], [985, 117], [1752, 79], [2150, 27], [2558, 37]]

@n = @gm * @gn
@voltagesSemi = @vs1; @voltages = @vs2
@d = @voltagesSemi.length * 1 + @voltages.length * 2

def semiDirectProduct(x1, y1, x2, y2)
  [(x1 + (@gr ** y1) * x2) % @gm, (y1 + y2) % @gn]
end

def cayleyGraph(vs1, vs2)
  strvol = "voltages: #{vs1}, semiV: #{vs2}"
  @e = (0...@n).map { |i| [] }

  @dupc = 0
  (0...@gm).each { |mi|
    (0...@gn).each { |ni|
      (vs1 + vs2).each { |mv, nv|
        mj, nj = semiDirectProduct(mi, ni, mv, nv)
        i = mi + @gm * ni
        j = mj + @gm * nj
        if @e[i].index(j) == nil # 片方向の枝は @e0 に
          addEdge(@e, i, j)
        else
          @dupc += 1
        end
      }
    }
  }
  d = vs1.length + 2 * vs2.length
  
  dist = (0...@n).to_a.map { @n }
  frontier = [0]
  dist[0] = 0
  while frontier.length > 0
    nextfrontier = []
    frontier.each { |i|
      di = dist[i]
      @e[i].each { |j|
        if dist[j] > di + 1
          dist[j] = di + 1
          nextfrontier << j
        end
      }
    }
    frontier = nextfrontier
  end
  n = dist.reduce(0) { |s, di|
    if di < @n
      s += 1
    else
      s
    end
  }
  k = dist.max
  l = Rational(dist.reduce(:+), n - 1)
  evc = (0..k).to_a.map { 0 }
  (0...@n).each { |i|
    evc[dist[i]] += 1
  }
  
  @e.map! { |ei| ei.sort }
  save(@e, @n, @d, k, l.to_f, @gm, @gn, @gr, vs1, vs2)
  puts "# n#{n}d#{d}-k#{k}l#{l.to_f}-gm#{@gm}gn#{@gn}gr#{@gr}, #{evc}]"
  puts "@voltagesSemi = #{vs1}; @voltages = #{vs2}"
  [n, d, k, l]
end

def addEdge(e, i, j)
  e[i] << j
  e[j] << i
end

def save(e, n, d, k, l, gm, gn, gr, vs1, vs2)
  return if e.length != n
  fnout = "n#{n}d#{d}-k#{k}l#{l.to_f}-gm#{gm}gn#{gn}gr#{gr}.edges"
  puts "write into
  #{fnout}"
  fo = open(fnout, 'w')
  vs1s = vs1.map { |v| "(#{v.join(',')})" }.join('')
  vs1s += '|' if vs1.length > 0 && vs2.length > 0
  vs2s = vs2.map { |v| "(#{v.join(',')})" }.join('')
  fo.puts "# Cayley graph"
  fo.puts "# Group Z_#{@gm} rtimes_#{@gn} Z_#{@gr}; Quotient B(#{vs1.length},#{vs2.length}) "
  fo.puts "# Voltages [#{vs1s}#{vs2s}]"
  (0...n).each { |i|
    e[i].sort.each { |j|
      next if i > j
      fo.puts "#{i} #{j}"
    }
  }
  fo.close
end

cayleyGraph(@voltagesSemi, @voltages)
